// picalc version 1.3 for TI-89 and TI-92 Plus

#define USE_LONGMUL_PATCH
#define USE_LONGDIV_PATCH
#define OPTIMIZE_ROM_CALLS
#define SAVE_SCREEN

#include <nostub.h>
#include <all.h>

#define base 10000  			// The lower the base,the more digits it can calculate but the
                    			// slower the program. The limit of the base depends on the
                    			// formula being used.  Must be a positive power of 10.

#define basedigits 4			// This must be set to log10(base)

#define ARC10       			// This can be set to ARC2, ARC3, ARC7, ARC18, ARC10 or it can be
                    			// omitted to use Machin's formula.  Default is ARC10

int _ti92plus,_ti89;			// Compile for both calcs

long terms=0,*pi=0,*temp=0,first=0,digits=0,overflow=0,screenfit=0,done=1;
char exitcheck=0,print=0;
unsigned long m=0;
volatile unsigned long time = 0;

void divide(long d)
{
	unsigned long c;
	long i=first;
	while(temp[i]==0) {
		i++;
		if(i==terms) return;
		first++;
	}
	c = (unsigned long) temp[i]%d;
	temp[i] = (unsigned long) temp[i]/d;
	for(i++;i<terms;i++) {
		temp[i] = (unsigned long) temp[i]+base*c;
		while(temp[i]==0) {
			i++;
			if(i==terms) return;
		}
		c = (unsigned long) temp[i]%d;
		temp[i] = (unsigned long) temp[i]/d;
	}
}

void divi(long d,char a,char b)
{
	unsigned long c;
	long i=first;
	m = (unsigned long) temp[i]/d;
	c = (unsigned long) temp[i]%d;
	pi[i] += a*b*m;
	for(i++;i<terms;i++) {
		m = (unsigned long) temp[i]+base*c;
		c = (unsigned long) m%d;
		m = (unsigned long) m/d;
		pi[i] += a*b*m;
	}
}

void arccot(int t,int b,char sign)
{
	long i,check=terms-1;
	first = 0;
	temp[0] = t;
	divide(b);
	for(i=0;i<terms;i++) pi[i] += sign*temp[i];
	for(i=1;;i+=2) {
		divide((long) b*b);
		divi((long) 2*i+1,-1,sign);
		if(first==check) return;
		divide((long) b*b);
		divi((long) 2*i+3,1,sign);
		if(first==check) return;
		if(OSCheckBreak()==TRUE) {
			exitcheck = 1;
			return;
		}
	}
}

void check()
{
	long x,i;
	for(i=terms-1;i>0;i--) {
		x = pi[i]/base;
		if(pi[i]<0 && pi[i]%base) x--;
		pi[i] -= base*x;
		pi[i-1] += x;
	}
}

void printit()
{
	long i,printed=0;
	int output[basedigits] = {};
	FILE *f_out;
	if(print) f_out = fopen("pi","w+t");
	clrscr();
	printf_xy(0,LCD_HEIGHT-6,"%ld.%ld sec",time/10,time%10);
	printf("%ld.",pi[0]);
	if(print && f_out != NULL) fprintf(f_out,"%ld.",pi[0]);
	if(digits==0) return;
	for(i=1;;i++) {
		int q;
		for(q=basedigits-1;q>=0;q--) {
			output[q] = pi[i]%10;
			pi[i] /= 10;
		}
		for(q=0;q<basedigits;q++) {
			printf("%d",output[q]);
			if(print && f_out != NULL) fprintf(f_out,"%d",output[q]);
			printed++;
			if(printed==digits) {
				if(print && f_out != NULL) fclose(f_out);
				return;
			}
			if(printed%10==0) printf(" ");
			if(printed%screenfit==0) ngetchx();
			if(printed%overflow==0) printf("\n   ");
		}
	}
}

void countdown()
{
	if(done) {
		time += (unsigned long) 1;
		printf_xy(0,LCD_HEIGHT-6,"%ld.%ld sec",time/10,time%10);
	}
}

void _main(void)
{
	ESI argptr = top_estack;
	int type = GetArgType(argptr);
	done = 1;
	exitcheck = time = print = 0;
	FontSetSys(F_4x6);
	if(CALCULATOR==0) {
		overflow = 30;
		screenfit = 450;
	}
	else {
		overflow = 50;
		screenfit = 1000;
	}
	if(type!=POSINT_TAG && type!=NEGINT_TAG) digits = 100;
	else digits = GetIntArg(argptr);
	clrscr();
	printf("Calculating %ld digits...\n",digits);
	terms = digits/basedigits+4;
	pi = (long *)calloc(terms,sizeof(long));
	temp = (long *)calloc(terms,sizeof(unsigned long));
	if(pi==NULL || temp==NULL) {
		printf("Not enough memory.\n");
		ngetchx();
		return;
	}
	OSVFreeTimer(2);
	OSVRegisterTimer(2,2,countdown);
	OSEnableBreak();
// The arccot function is in the form arccot(a,b,c) which is equivalent to a*arccot(b) with sign c
// You may add any other formula you wish

#if defined ARC2
	// Euler's formula
	arccot(4,2,1);
	if(exitcheck != 0) goto done;
	arccot(4,3,1);
	if(exitcheck != 0) goto done;
	
#elif defined ARC3
	// Hermann's formula
	arccot(8,2,1);
	if(exitcheck != 0) goto done;
	arccot(4,7,-1);
	if(exitcheck != 0) goto done;
	
#elif defined ARC7
	// Hutton's formula
	arccot(8,3,1);
	if(exitcheck != 0) goto done;
	arccot(4,7,1);
	if(exitcheck != 0) goto done;
	
#elif defined ARC18
	// Gauss's formula
	arccot(48,18,1);
	if(exitcheck != 0) goto done;
	arccot(32,57,1);
	if(exitcheck != 0) goto done;
	arccot(20,239,-1);
	if(exitcheck != 0) goto done;
	
#elif defined ARC10
	// Klingenestierna's formula (this is fastest)
	arccot(32,10,1);
	if(exitcheck != 0) goto done;
	arccot(4,239,-1);
	if(exitcheck != 0) goto done;
	arccot(16,515,-1);
	if(exitcheck != 0) goto done;
	
#elif
	// Machin's formula 
	arccot(16,5,1);
	if(exitcheck != 0) goto done;
	arccot(4,239,-1);
	if(exitcheck != 0) goto done;
#endif

	check();
	done = 0;
	printf("Save result to file (y/n): ");
	terms = ngetchx();
	if(terms=='y') print=1;
	printit();
	ngetchx();
done:
	OSVFreeTimer(2);
	OSDisableBreak();
	OSClearBreak();
	free(pi);
	free(temp);
	return;
}

